function x_hat = MinimaxRegretEstimator(y, X, Z_x, Y, Z_y, K_z, K_y)
% Input:
%   y - an N x 1 measurement vecor on which the estimator is applied
%   X - an M x L matrix containing L examples of signals x (paired with Z_x)
%   Z_x - a Q x L matrix containing L examples of instruments z (paired with X) 
%   Y - an N x P matrix containing P examples of measurements y (paired with Z_y)
%   Z_y - a Q x P matrix containing P examples of instruments z (paired with Y) 
%   K_z - isotrpoic kernel (function handle) accepting a distance d(z_i,z_j) and returning a positive scalar
%   K_y - isotrpoic kernel (function handle) accepting a distance d(y_i,y_j) and returning a positive scalar
% Output:
%   x_hat - the estimate corresponding to the measurements y

sum_weights_outer = 0;
x_hat = 0;
hw = waitbar(0, 'wait...');

P = size(X, 2);
L = size(Y, 2);

for j = 1:L
    waitbar(j /L, hw);
    E_X_given_Zy_j = 0;
    sum_weights_inner = 0;
    for i = 1:P
        weight_inner = feval(K_z, Z_y(:, j) - Z_x(:, i));
        E_X_given_Zy_j = E_X_given_Zy_j + X(:, i) * weight_inner;
        sum_weights_inner = sum_weights_inner + weight_inner;
    end
    E_X_given_Zy_j = E_X_given_Zy_j / sum_weights_inner;
    
    weight_outer = feval(K_y, y - Y(:, j));
    x_hat = x_hat + E_X_given_Zy_j * weight_outer;
    sum_weights_outer = sum_weights_outer + weight_outer;
end
x_hat = x_hat / sum_weights_outer;
close(hw);